"""
An exceptionally lousy site spider
Ken Kinder <ken@kenkinder.com>

Updated for newparallel by Min Ragan-Kelley <benjaminrk@gmail.com>

This module gives an example of how the task interface to the
IPython controller works.  Before running this script start the IPython controller
and some engines using something like::

    ipcluster start -n 4
"""
from __future__ import print_function

import sys
import ipyparallel as ipp
import time
import bs4 # this isn't necessary, but it helps throw the dependency error earlier

try:
    raw_input
except NameError:
    raw_input = input

def fetchAndParse(url, data=None):
    import requests
    try:
        from urllib.parse import urljoin
    except ImportError:
        from urlparse import urljoin
    import bs4
    links = []
    r = requests.get(url, data=data)
    r.raise_for_status()
    if 'text/html' in r.headers.get('content-type'):
        doc = bs4.BeautifulSoup(r.text, "html.parser")
        for node in doc.findAll('a'):
            href = node.get('href', None)
            if href:
                links.append(urljoin(url, href))
    return links

class DistributedSpider(object):

    # Time to wait between polling for task results.
    pollingDelay = 0.5

    def __init__(self, site):
        self.client = ipp.Client()
        self.view = self.client.load_balanced_view()
        self.mux = self.client[:]

        self.allLinks = []
        self.linksWorking = {}
        self.linksDone = {}

        self.site = site

    def visitLink(self, url):
        if url not in self.allLinks:
            self.allLinks.append(url)
            if url.startswith(self.site):
                print('    ', url)
                self.linksWorking[url] = self.view.apply(fetchAndParse, url)

    def onVisitDone(self, links, url):
        print(url + ':')
        self.linksDone[url] = None
        del self.linksWorking[url]
        for link in links:
            self.visitLink(link)

    def run(self):
        self.visitLink(self.site)
        while self.linksWorking:
            print(len(self.linksWorking), 'pending...')
            self.synchronize()
            time.sleep(self.pollingDelay)

    def synchronize(self):
        for url, ar in list(self.linksWorking.items()):
            # Calling get_task_result with block=False will return None if the
            # task is not done yet.  This provides a simple way of polling.
            try:
                links = ar.get(0)
            except ipp.error.TimeoutError:
                continue
            except Exception as e:
                self.linksDone[url] = None
                del self.linksWorking[url]
                print('%s: %s' % (url, e))
            else:
                self.onVisitDone(links, url)

def main():
    if len(sys.argv) > 1:
        site = sys.argv[1]
    else:
        site = raw_input('Enter site to crawl: ')
    distributedSpider = DistributedSpider(site)
    distributedSpider.run()

if __name__ == '__main__':
    main()



#!/usr/bin/env python
# coding: utf-8

# Copyright (c) IPython Development Team.
# Distributed under the terms of the Modified BSD License.

from __future__ import print_function

# the name of the project
name = 'ipyparallel'

#-----------------------------------------------------------------------------
# Minimal Python version sanity check
#-----------------------------------------------------------------------------

import sys

v = sys.version_info
if v[:2] < (2, 7) or (v[0] >= 3 and v[:2] < (3, 4)):
    error = "ERROR: %s requires Python version 2.7 or 3.4 or above." % name
    print(error, file=sys.stderr)
    sys.exit(1)

PY3 = (sys.version_info[0] >= 3)

#-----------------------------------------------------------------------------
# Add test command
#-----------------------------------------------------------------------------


from setuptools.command.bdist_egg import bdist_egg


class bdist_egg_disabled(bdist_egg):
    """Disabled version of bdist_egg

    Prevents setup.py install performing setuptools' default easy_install,
    which it should never ever do.
    """
    def run(self):
        sys.exit("Aborting implicit building of eggs. Use `pip install .` to install from source.")

#-----------------------------------------------------------------------------
# get on with it
#-----------------------------------------------------------------------------

import os
from glob import glob

from setuptools import setup

pjoin = os.path.join
here = os.path.abspath(os.path.dirname(__file__))
pkg_root = pjoin(here, name)

packages = []
for d, _, _ in os.walk(pjoin(here, name)):
    if os.path.exists(pjoin(d, '__init__.py')):
        packages.append(d[len(here)+1:].replace(os.path.sep, '.'))

package_data = {'ipyparallel.nbextension': [pjoin('static', '*')]}

data_files = [
    (
        'etc/jupyter/jupyter_notebook_config.d',
        [pjoin('etc', 'ipyparallel-serverextension.json')],
    ),
    (
        'etc/jupyter/nbconfig/tree.d',
        [pjoin('etc', 'ipyparallel-nbextension.json')],
    ),
    (
        'share/jupyter/nbextensions/ipyparallel',
        glob(pjoin('ipyparallel', 'nbextension', 'static', '*')),
    ),
]

version_ns = {}
with open(pjoin(here, name, '_version.py')) as f:
    exec(f.read(), {}, version_ns)


setup_args = dict(
    name=name,
    version=version_ns["__version__"],
    packages=packages,
    package_data=package_data,
    description="Interactive Parallel Computing with IPython",
    long_description="""Use multiple instances of IPython in parallel, interactively.

    See https://ipyparallel.readthedocs.io for more info.
    """,
    author="IPython Development Team",
    author_email="ipython-dev@scipy.org",
    url="http://ipython.org",
    license="BSD",
    platforms="Linux, Mac OS X, Windows",
    keywords=["Interactive", "Interpreter", "Shell", "Parallel"],
    classifiers=[
        "Intended Audience :: Developers",
        "Intended Audience :: System Administrators",
        "Intended Audience :: Science/Research",
        "License :: OSI Approved :: BSD License",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
    ],
    cmdclass={
        "bdist_egg": bdist_egg if "bdist_egg" in sys.argv else bdist_egg_disabled,
    },
    data_files=data_files,
    install_requires=[
        "ipython_genutils",
        "decorator",
        "pyzmq>=13",
        "traitlets>=4.3",
        "ipython>=4",
        "jupyter_client",
        "ipykernel>=4.4",
        "tornado>=4",
        "python-dateutil>=2.1",
    ],
    python_requires=">=3.5",
    extras_require={
        "nbext": ["notebook"],
        "test": ["pytest", "pytest-cov", "ipython[test]", "testpath", "mock"],
    },
    entry_points={
        "console_scripts": [
            "ipcluster = ipyparallel.apps.ipclusterapp:launch_new_instance",
            "ipcontroller = ipyparallel.apps.ipcontrollerapp:launch_new_instance",
            "ipengine = ipyparallel.apps.ipengineapp:launch_new_instance",
        ]
    },
)


if __name__ == "__main__":
    setup(**setup_args)
